@@ -6,9 +6,9 @@ library(soundgen)
 s = soundgen(play = playback)  # default sound: a short [a] by a male speaker
 # 's' is a numeric vector - the waveform. You can save it, play it, plot it, ...
 
-# names(presets)  # speakers
-# names(presets$Chimpanzee)  # calls per speaker
-s = eval(parse(text = presets$Chimpanzee$Scream_conflict))  # screaming chimp
+# names(presets)  # speakers in the preset library
+# names(presets$Chimpanzee)  # presets per speaker
+s = eval(parse(text = presets$Chimpanzee$Scream_conflict))  # a screaming chimp
 # playme(s)
 
 ## ------------------------------------------------------------------------
@@ -40,12 +40,11 @@ sound = soundgen(sylLen = 2000, play = playback,
 
 ## ----fig.width = 5, fig.height = 5---------------------------------------
 s = soundgen(nSyl = 5, sylLen = 200, pauseLen = 140, plot = TRUE, play = playback,
-                 pitchAnchors = data.frame(time = c(0, 0.65, 1), 
-                                           value = c(977, 1540, 826)),
-                 pitchAnchorsGlobal = data.frame(time = c(0, .5, 1), 
-                                                 value = c(-6, 7, 0)))
-# NB: pitchAnchorsGlobal = c(-6, 7, 0) produces exactly the same result, 
-# but only the dataframe format is compatible with the app
+             pitchAnchors = data.frame(time = c(0, 0.65, 1), 
+                                       value = c(977, 1540, 826)),
+             pitchAnchorsGlobal = data.frame(time = c(0, .5, 1), 
+                                             value = c(-6, 7, 0)))
+# pitchAnchorsGlobal = c(-6, 7, 0) is equivalent, since time steps are equal
 
 ## ------------------------------------------------------------------------
 # the sound is a bit different each time, because temperature is above zero
@@ -60,13 +59,13 @@ s = soundgen(repeatBout = 3, nSyl = 3, temperature = .3, play = playback,
              tempEffects = list(sylLenDep = 0, formDrift = .8))
 
 ## ------------------------------------------------------------------------
-mf = c(-1,  # male: 100% lower F0, 25% lower formants, 25% longer vocal tract
+mf = c(-1,  # male: 100% lower f0, 25% lower formants, 25% longer vocal tract
        0,   # neutral (default)
-       1)   # female: 100% higher F0, 25% higher formants, 25% shorter vocal tract
+       1)   # female: 100% higher f0, 25% higher formants, 25% shorter vocal tract
 # See e.g. http://www.santiagobarreda.com/vignettes/v1/v1.html
 
 for (i in mf) {
-  s = soundgen(maleFemale = i, formants = NA, vocalTract = 25, play = playback)
+  s = soundgen(maleFemale = i, formants = NA, vocalTract = 17, play = playback)
   # Since `formants` are not specified, but temperature is above zero, a 
   # schwa-like sound with approximately equidistant formants is generated using
   # `vocalTract` (cm) to calculate the expected formant dispersion.
@@ -87,16 +86,18 @@ for (i in cb) {
 # in the oscillogram under the spectrogram), and there is an overall fade-out
 # over the entire bout
 s = soundgen(nSyl = 4, plot = TRUE, osc = TRUE, play = playback,
-             amplAnchors = data.frame(time = c(0, .5, 1), 
+             amplAnchors = data.frame(time = c(0, .3, 1),  # unequal time steps
                                       value = c(120, 100, 120)),
-             amplAnchorsGlobal = data.frame(time = c(0, 1), 
-                                            value = c(120, 0)))
+             amplAnchorsGlobal = c(120, 0))  # equal time steps, so shorthand is ok
 
 ## ----fig.width = 5, fig.height = 5---------------------------------------
 s = soundgen(sylLen = 1000, formants = NA,
-             amDep = 50,   # halves the amplitude at troughs (0% = none, 100% = max)
-             amFreq = 35,  # amplitude modulation with frequency 35 Hz
-             amShape = 0,  # 0 = close to sine, -1 = notches, +1 = clicks
+             # set the depth of AM (0% = none, 100% = max)
+             amDep = c(0, 100),   
+             # set AM frequency in Hz (vectorized)
+             amFreq = c(50, 25),  
+             # set the shape: 0 = close to sine, -1 = notches, +1 = clicks
+             amShape = 0,  
              plot = TRUE, osc = TRUE, play = playback)
 
 ## ------------------------------------------------------------------------
@@ -104,6 +105,13 @@ s = soundgen(formants = 'ai', play = playback)
 s = soundgen(formants = 'aaai', play = playback)
 
 ## ------------------------------------------------------------------------
+# shorthand specification with three stationary formants
+formants = c(300, 2500, 3200)
+
+# shorthand specification with two moving formants
+formants = list(f1 = c(300, 900), f2 = c(2500, 1500))
+
+# full specification with two moving formants and non-default amplitude and bandwidth
 formants = list(
   f1 = data.frame(time = c(0, 1), 
                   freq = c(300, 900), 
@@ -115,43 +123,59 @@ formants = list(
                   width = c(0, 240)))
 
 ## ----fig.width = 4, fig.height = 4---------------------------------------
+# a simple filter with 3 formants at 500, 1300 and 2900 Hz
+se = getSpectralEnvelope(nr = 1024, nc = 1, formants = c(500, 1300, 2900))
+freqs = seq(1, 16000 / 2, length.out = 1024)
+plot(freqs, log10(se) * 20, type = 'l', ylab = 'dB', xlab = 'Frequency, Hz')
+
+## ------------------------------------------------------------------------
+estimateVTL(formants = c(500, 1500, 2500))  # 17.7 cm
+estimateVTL(formants = c(300, 1100, 2000))  # 22.8 cm
+
+## ----fig.show = "hold", fig.width = 7, fig.height = 4--------------------
+s1 = soundgen(formants = c(800, 1200), play = playback, plot = TRUE)
+s1 = soundgen(formants = c(800, 1200), formantDepStoch = 0,
+              play = playback, plot = TRUE)
+
+## ----fig.show = "hold", fig.width = 7, fig.height = 4--------------------
+s1 = soundgen(formants = NULL, vocalTract = 24, play = playback, plot = TRUE)
+s1 = soundgen(formants = NULL, vocalTract = 12, play = playback, plot = TRUE)
+
+## ----fig.width = 4, fig.height = 4---------------------------------------
+# plotting directly from getSpectralEnvelope() in spectrogram form
 s = getSpectralEnvelope(nr = 1024,  # freq bins in FFT frame (window_length / 2)
-                        nc = 50,    # time 
+                        nc = 50,    # time bins
                         samplingRate = 16000, 
                         formants = formants,
                         plot = TRUE, 
                         dur = 1500,   # just an example
                         colorTheme = 'seewave',
-                        rolloffLip = 6) 
-# Note that lip radiation is also specified here, as "rolloffLip" (dB). 
+                        lipRad = 6) 
+# Note that lip radiation is also specified here (dB). 
 # This has the effect of amplifying higher frequencies to mimic lip radiation. 
 
-## ------------------------------------------------------------------------
-s = soundgen(formants = formants, play = playback)
-
-## ----fig.show = "hold", fig.width = 5, fig.height = 5--------------------
-s = soundgen(sylLen = 1500, play = playback,
-             pitchAnchors = data.frame(time = c(0, 1), value = c(140, 140)), 
-             formants = list(
+## ----fig.show = "hold", fig.width = 5, fig.height = 4--------------------
+formants = list(
                f1   = data.frame(time = c(0, 1), freq = c(880, 900), 
-                                 amp = c(40,20), width = c(80,120)), 
+                                 amp = c(30, 20), width = c(80, 120)), 
                f1.5 = data.frame(time = c(0, 1), freq = 600, 
-                                 amp = c(0, 30), width= 80), 
+                                 amp = c(0, 30), width = 80),   # additional pole
                f1.7 = data.frame(time=c(0, 1), freq = 750, 
-                                 amp = c(0, -80), width = 80), 
+                                 amp = c(0, -30), width = 80), # zero
                f2   = data.frame(time = c(0, 1), freq = c(1480, 1250), 
-                                 amp = c(40, 20), width = c(120, 200)), 
+                                 amp = c(30, 20), width = c(120, 200)), 
                f3   = data.frame(time=c(0, 1), freq = c(2900, 3100), 
-                                 amp = 40, width = 200)))
+                                 amp = 25, width = 200))
+# se = getSpectralEnvelope(nr = 512, nc = 100, formants = formants, plot = TRUE)
+s = soundgen(sylLen = 1500, play = playback, 
+             pitchAnchors = 140, formants = formants)
 spectrogram(s, samplingRate = 16000, ylim = c(0, 4), contrast = .5, 
      windowLength = 10, step = 5, colorTheme = 'seewave')
 # long-term average spectrum (less helpful for moving formants but very good for stationary):
 # seewave::meanspectrogram(s, f = 16000, wl = 256)  
 
 ## ----fig.width = 7, fig.height = 5---------------------------------------
-s = soundgen(sylLen = 700, play = playback,
-             pitchAnchors = data.frame(time = c(0, 1), 
-                                       value = c(140, 140)), 
+s = soundgen(sylLen = 1200, play = playback, pitchAnchors = 140, 
              mouthAnchors = list(time = c(0, .3, .75, 1), 
                                  value = c(0, 0, .7, 0)))
 spectrogram(s, samplingRate = 16000, 
@@ -161,62 +185,76 @@ spectrogram(s, samplingRate = 16000,
 
 ## ----fig.show = "hold", fig.width = 4, fig.height = 4--------------------
 # strong F0, rolloff with a "shoulder"
-r = getRolloff(rolloff = -20, rolloffOct = -3,
+r = getRolloff(rolloff = c(-5, -20),  # rolloff parameters are vectorized
+               rolloffOct = -3,
                rolloffParab = -10, rolloffParabHarm = 13, 
                pitch_per_gc = c(170, 340), plot = TRUE)
 
 # to generate the corresponding sound:
-s = soundgen(rolloff = -20, rolloffOct = -3, play = playback,
+s = soundgen(sylLen = 1000, rolloff = c(-5, -20), rolloffOct = -3,
              rolloffParab = -10, rolloffParabHarm = 13,
-             pitchAnchors = data.frame(time = c(0, 1), value = c(170, 340)))
+             pitchAnchors = c(170, 340),  play = playback)
+
+## ------------------------------------------------------------------------
+# Not a good idea: samplingRate is too low
+s1 = soundgen(pitchAnchors = c(1500, 800), glottisAnchors = 75, 
+              samplingRate = 16000, play = playback)
+
+# This sounds better but takes a long time to synthesize:
+s2 = suppressWarnings(soundgen(pitchAnchors = c(1500, 800), glottisAnchors = 75, 
+              samplingRate = 80000, play = playback, 
+              invalidArgAction = 'ignore'))
+# NB: invalidArgAction = 'ignore' forces a "weird" samplingRate value
+# to be accepted without question
+
+# Now this is what this feature is for: vocal fry
+s3 = soundgen(sylLen = 1500, pitchAnchors = c(75, 40), 
+              glottisAnchors = c(0, 700), 
+              samplingRate = 16000, play = playback)
+plot(s3, type = 'l')
 
 ## ----fig.show = "hold", fig.width = 3, fig.height = 3--------------------
 s1 = soundgen(subFreq = 400, subDep = 150, nonlinBalance = 100,
               jitterDep = 0, shimmerDep = 0, temperature = 0, 
-              sylLen = 500, pitchAnchors = data.frame(time=c(0, 1), 
-                                                      value = c(800, 900)),
-              play = playback, plot = TRUE)
+              sylLen = 500, pitchAnchors = c(800, 900),
+              play = playback, plot = TRUE, ylim = c(0, 3))
 s2 = soundgen(subFreq = 400, subDep = 400, nonlinBalance = 100,
               jitterDep = 0, shimmerDep = 0, temperature = 0, 
-              sylLen = 500, pitchAnchors = data.frame(time=c(0, 1), 
-                                                      value = c(800, 900)),
-              play = playback, plot = TRUE)
+              sylLen = 500, pitchAnchors = c(800, 900),
+              play = playback, plot = TRUE, ylim = c(0, 3))
 
 ## ----fig.width = 5, fig.height = 5---------------------------------------
 s = soundgen(subFreq = 75, subDep = 130, nonlinBalance = 100,
              jitterDep = 0, shimmerDep = 0, temperature = 0, 
-             sylLen = 800, plot = TRUE, play = playback,
+             sylLen = 800, formants = NULL, play = playback,
              pitchAnchors = data.frame(time=c(0, .3, .9, 1), 
                                        value = c(1200, 1547, 1487, 1154)))
+spectrogram(s, 16000, windowLength = 50, ylim = c(0, 5), contrast = .7)
 
 ## ------------------------------------------------------------------------
 # To get jitter without subharmonics, set `temperature = 0, subDep = 0` 
 # and specify the required jitter depth and period
 s1 = soundgen(jitterLen = 50, jitterDep = 1,  # shaky voice
               sylLen = 1000, subDep = 0, nonlinBalance = 100,
-              pitchAnchors = data.frame(time = c(0, 1), 
-                                        value = c(150, 170)),
-              play = playback)
-s2 = soundgen(jitterLen = 1, jitterDep = 1,  # harsh voice
+              pitchAnchors =  c(150, 170), play = playback)
+s2 = soundgen(jitterLen = 1, jitterDep = 1,   # harsh voice
               sylLen = 1000, subDep = 0, nonlinBalance = 100,
-              pitchAnchors = data.frame(time = c(0, 1), 
-                                        value = c(150, 170)),
-              play = playback)
+              pitchAnchors =  c(150, 170), play = playback)
 
 ## ------------------------------------------------------------------------
-s = soundgen(repeatBout = 2, sylLen = 140, pauseLen = 100, vocalTract = 8,
-             pitchAnchors = list(time = c(0, 0.52, 1), value = c(559, 785, 557)), 
-             nonlinBalance = 100, jitterDep = 1, subDep = 60, play = playback,
-             mouthAnchors = list(time = c(0, 0.5, 1), value = c(0, 0.5, 0)))
+s = soundgen(repeatBout = 2, sylLen = 140, pauseLen = 100, 
+             vocalTract = 8, formants = NULL,
+             pitchAnchors = c(559, 785, 557), mouthAnchors =  c(0, 0.5, 0),
+             nonlinBalance = 100, jitterDep = 1, subDep = 60, play = playback) 
 
 ## ------------------------------------------------------------------------
 # 5-Hz vibrato 1 semitone in depth
 s1 = soundgen(vibratoDep = 1, vibratoFreq = 5, sylLen = 1000, play = playback,
-              pitchAnchors = data.frame(time = c(0, 1), value = c(300, 280)))
+              pitchAnchors = c(300, 280))
 
 # slower (3 Hz) and deeper (3 semitones) vibrato
 s2 = soundgen(vibratoDep = 3, vibratoFreq = 3, sylLen = 1000, play = playback,
-              pitchAnchors = data.frame(time = c(0, 1), value = c(300, 280)))
+              pitchAnchors = c(300, 280))
 
 ## ------------------------------------------------------------------------
 s = soundgen(noiseAnchors = data.frame(time = c(0, 500), value = c(-40, 20)),
@@ -232,44 +270,25 @@ s = soundgen(noiseAnchors = data.frame(time = c(0, 500), value = c(-40, 20)),
                            amp = 50, 
                            width = 1000)
            ), 
-         rolloffNoise = 0,
+         rolloffNoise = 0, pitchAnchors = NA,
          sylLen = 500, play = playback, plot = TRUE)
 
 ## ----fig.width = 7, fig.height = 5---------------------------------------
 s = soundgen(nSyl = 2, sylLen = 120, pauseLen = 120, 
              temperature = 0, rolloffNoise = -5, 
              noiseAnchors = data.frame(time = c(39, 56, 209), 
-                                       value = c(-120, -10, -120)),
-             formants = list(
-               f1 = data.frame(time = c(0, 1), 
-                               freq = c(860, 530), 
-                               amp = 30, 
-                               width = c(120, 50)
-               ), 
-               f2 = data.frame(time = c(0, 1), 
-                               freq = c(1280, 2400), 
-                               amp = 40, 
-                               width = c(120, 300))),
-             formantsNoise = list(
-               f1 = data.frame(time = 0, 
-                               freq = 420, 
-                               amp = 20, 
-                               width = 150),
-               f2 = data.frame(time = 0, 
-                               freq = 1200, 
-                               amp = 50, 
-                               width = 250)
-             ),
+                                       value = c(-80, -10, -50)),
+             formants = list(f1 = c(860, 530),  f2 = c(1280, 2400)),
+             formantsNoise = c(420, 1200),
              plot = TRUE, osc = TRUE, play = playback)
 
 ## ----fig.width = 5, fig.height = 4---------------------------------------
 a = getSmoothContour(anchors = data.frame(time = c(-50, 200, 300), 
-                                          value = c(-120, 20, -120)),
-                     voiced = 200, plot = TRUE, ylim = c(-120, 40), main = '')
+                                          value = c(-80, 20, -80)),
+                     voiced = 200, plot = TRUE, ylim = c(-80, 40), main = '')
 
 ## ------------------------------------------------------------------------
 s1 = soundgen(vocalTract = 17.5,  # ~human throat (17.5 cm)
-              formants = NULL, attackLen = 200, play = playback,
               noiseAnchors = list(time = c(-8, 813), value = c(40, 40)))
 
 s2 = soundgen(vocalTract = 30,    # a large animal
@@ -277,6 +296,11 @@ s2 = soundgen(vocalTract = 30,    # a large animal
               noiseAnchors = list(time = c(-8, 813), value = c(40, 40)))
 # NB: voiced component not generated, since noiseAnchors$value >= 40 dB
 
+## ----fig.width = 4, fig.height = 3---------------------------------------
+s3 = soundgen(vocalTract = 17.5, formantsNoise = c(1000, 2000),  
+              noiseAnchors = list(time = c(-8, 813), value = 40),
+              play = playback, plot = TRUE)
+
 ## ----fig.show = "hold", fig.width = 7, fig.height = 3--------------------
 par(mfrow = c(1, 2))
 sound1 = sin(2 * pi * 1:5000 * 100 / 16000) # pure tone, 100 Hz
@@ -295,10 +319,17 @@ plot(comb2[4000:5500], type = 'l')  # gradual transition
 # spectrogram(comb2, 16000)
 par(mfrow = c(1, 1))
 
+## ----fig.width = 5, fig.height = 5---------------------------------------
+cow1 = soundgen(sylLen = 1400, pitchAnchors = list(time = c(0, 11/14, 1), value = c(75, 130, 200)), temperature = 0.1, rolloff = -6, rolloffOct = -3, rolloffParab = 12, mouthOpenThres = 0.6, formants = NULL, vocalTract = 36.5, mouthAnchors = list(time = c(0, 0.82, 1), value = c(0.6, 0, 1)), noiseAnchors = list(time = c(0, 1400), value = c(-25, -25)), rolloffNoise = -4, addSilence = 0)
+cow2 = soundgen(sylLen = 310, pitchAnchors = list(time = c(0, 1), value = c(359, 359)), temperature = 0.05, nonlinBalance = 100, jitterDep = 1.3, attackLen = 0, rolloff = -6, rolloffOct = -3, rolloffKHz = -0, formants = NULL, vocalTract = 36.5, subFreq = 150, subDep = 70, noiseAnchors = list(time = c(0, 26, 317, 562), value = c(-80, -23, -22, -80)), rolloffNoise = -6, addSilence = 0)
+s = crossFade(cow1, cow2, samplingRate = 16000, crossLen = 150)
+# playme(s, 16000)
+spectrogram(s, 16000, osc=T, ylim = c(0, 4))
+
 ## ----fig.show = "hold", fig.width = 5, fig.height = 5--------------------
 sound1 = soundgen(sylLen = 700, pitchAnchors = 250:180, formants = 'aaao', 
                   addSilence = 100, play = playback)
-sound2 = soundgen(nSyl = 2, sylLen = 150, pitchAnchors = 4300:2200, attackLen = 10, 
+sound2 = soundgen(nSyl = 2, sylLen = 150, pitchAnchors = 4300:2200, attackLen = 10,
                   formants = NA, temperature = 0, addSilence = 0, play = playback)
 
 insertionTime = .1 + .15  # silence + 150 ms
@@ -309,7 +340,8 @@ comb = addVectors(sound1,
                   insertionPoint = insertionPoint)
 # NB: soundgen softens attack by default, so no clicks are produced by overlapping
 # playme(comb)
-spectrogram(comb, 16000, windowLength = 10, ylim = c(0, 5), contrast = .5, colorTheme = 'seewave')
+spectrogram(comb, 16000, windowLength = 10, ylim = c(0, 5), 
+            contrast = .5, colorTheme = 'seewave')
 
 ## ----fig.show = "hold", fig.width = 7, fig.height = 3--------------------
 a = data.frame(time=c(0, .2, .9, 1), value=c(100, 110, 180, 110))
@@ -334,8 +366,9 @@ m = morph(formula1 = list(repeatBout = 2),
 
 ## ------------------------------------------------------------------------
 target = soundgen(repeatBout = 3, sylLen = 120, pauseLen = 70,
-                  pitchAnchors = data.frame(time = c(0, 1), value = c(300, 200)),
+                  pitchAnchors =c(300, 200),
                   rolloff = -5, play = playback)  # we hope to reproduce this sound
+# playme(target)
 
 m1 = matchPars(target = target,
                samplingRate = 16000,
@@ -347,12 +380,12 @@ cand1 = do.call(soundgen, c(m1$pars, list(play = playback, temperature = 0)))
 
 ## ---- eval = FALSE-------------------------------------------------------
 #  call('soundgen', m1$pars)
-#  # remove "list(...)" to get your call to soundgen():
-#  soundgen(samplingRate = 16000, nSyl = 3, sylLen = 79, pauseLen = 114,
-#      pitchAnchors = list(time = c(0, 0.5, 1), value = c(274, 253, 216)),
-#      formants = list(f1 = list(time = 0, freq = 821, amp = 30,  width = 122),
-#                      f2 = list(time = 0, freq = 1266, amp = 30, width = 36),
-#                      f3 = list(time = 0, freq = 2888, amp = 30, width = 117)))
+#  # copy-paste from the console and remove "list(...)" to get your call to soundgen():
+#  # soundgen(samplingRate = 16000, nSyl = 3, sylLen = 79, pauseLen = 114,
+#  #     pitchAnchors = list(time = c(0, 0.5, 1), value = c(274, 253, 216)),
+#  #     formants = list(f1 = list(freq = 821, width = 122),
+#  #                     f2 = list(freq = 1266, width = 36),
+#  #                     f3 = list(freq = 2888, width = 117)))
 
 ## ---- eval = FALSE-------------------------------------------------------
 #  m2 = matchPars(target = target,
